@@ -1,5 +1,6 @@
 from conan import conan_version
 
+
 def cyclonedx_1_4(conanfile, name=None, add_build=False, add_tests=False, **kwargs):
     """
     (Experimental) Generate cyclone 1.4 SBOM with JSON format
@@ -28,26 +29,30 @@ def cyclonedx_1_4(conanfile, name=None, add_build=False, add_tests=False, **kwar
     from datetime import datetime, timezone
     graph = conanfile.subgraph
 
-    has_special_root_node = not (getattr(graph.root.ref, "name", False) and getattr(graph.root.ref, "version", False) and getattr(graph.root.ref, "revision", False))
+    has_special_root_node = not (getattr(graph.root.ref, "name", False)
+                                 and getattr(graph.root.ref, "version", False)
+                                 and getattr(graph.root.ref, "revision", False))
     special_id = str(uuid.uuid4())
 
     name_default = getattr(graph.root.ref, "name", False) or "conan-sbom"
-    name_default += f"/{graph.root.ref.version}" if bool(getattr(graph.root.ref, "version", False)) else ""
-    nodes = [node for node in graph.nodes if (node.context == "host" or add_build) and (not node.test or add_tests)]
+    name_default += f"/{graph.root.ref.version}" if getattr(graph.root.ref, "version", False) else ""
+
+    nodes = [node for node in graph.nodes if should_add_node(node, add_build, add_tests)]
     if has_special_root_node:
         nodes = nodes[1:]
 
     dependencies = []
     if has_special_root_node:
         deps = {"ref": special_id,
-                "dependsOn": [f"pkg:conan/{d.dst.name}@{d.dst.ref.version}?rref={d.dst.ref.revision}"
-                              for d in graph.root.edges]}
+                "dependsOn": [_calculate_bomref(d.dst) for d in graph.root.edges
+                              if should_add_node(d.dst, add_build, add_tests)]}
         dependencies.append(deps)
     for c in nodes:
-        deps = {"ref": f"pkg:conan/{c.name}@{c.ref.version}?rref={c.ref.revision}"}
-        dep = [d for d in c.edges if (d.dst.context == "host" or add_build) and (not d.dst.test or add_tests)]
+        deps = {"ref": _calculate_bomref(c)}
+        dep = [d for d in c.edges if should_add_node(d.dst, add_build, add_tests)]
 
-        depends_on = [f"pkg:conan/{d.dst.name}@{d.dst.ref.version}?rref={d.dst.ref.revision}" for d in dep]
+        depends_on = [_calculate_bomref(d.dst) for d in dep
+                      if should_add_node(d.dst, add_build, add_tests)]
         if depends_on:
             deps["dependsOn"] = depends_on
         dependencies.append(deps)
@@ -55,7 +60,7 @@ def cyclonedx_1_4(conanfile, name=None, add_build=False, add_tests=False, **kwar
     sbom_cyclonedx_1_4 = {
         **({"components": [{
             "author": node.conanfile.author or "Unknown",
-            "bom-ref": f"pkg:conan/{node.name}@{node.ref.version}?rref={node.ref.revision}",
+            "bom-ref": _calculate_bomref(node),
             "description": node.conanfile.description,
             **({"externalReferences": [{
                 "type": "website",
@@ -71,7 +76,7 @@ def cyclonedx_1_4(conanfile, name=None, add_build=False, add_tests=False, **kwar
         "metadata": {
             "component": {
                 "author": conanfile.author or "Unknown",
-                "bom-ref": special_id if has_special_root_node else f"pkg:conan/{conanfile.name}@{conanfile.ref.version}?rref={conanfile.ref.revision}",
+                "bom-ref": special_id if has_special_root_node else _calculate_bomref(conanfile),
                 "name": name if name else name_default,
                 "type": "application" if conanfile.package_type == "application" else "library",
             },
@@ -91,6 +96,7 @@ def cyclonedx_1_4(conanfile, name=None, add_build=False, add_tests=False, **kwar
     }
     return sbom_cyclonedx_1_4
 
+
 def cyclonedx_1_6(conanfile, name=None, add_build=False, add_tests=False, **kwargs):
     """
     (Experimental) Generate cyclone 1.6 SBOM with JSON format
@@ -119,26 +125,31 @@ def cyclonedx_1_6(conanfile, name=None, add_build=False, add_tests=False, **kwar
     from datetime import datetime, timezone
     graph = conanfile.subgraph
 
-    has_special_root_node = not (getattr(graph.root.ref, "name", False) and getattr(graph.root.ref, "version", False) and getattr(graph.root.ref, "revision", False))
+    has_special_root_node = not (getattr(graph.root.ref, "name", False)
+                                 and getattr(graph.root.ref, "version", False)
+                                 and getattr(graph.root.ref, "revision", False))
     special_id = str(uuid.uuid4())
 
     name_default = getattr(graph.root.ref, "name", False) or "conan-sbom"
-    name_default += f"/{graph.root.ref.version}" if bool(getattr(graph.root.ref, "version", False)) else ""
-    nodes = [node for node in graph.nodes if (node.context == "host" or add_build) and (not node.test or add_tests)]
+    name_default += f"/{graph.root.ref.version}" if getattr(graph.root.ref, "version", False) else ""
+
+    nodes = [node for node in graph.nodes if should_add_node(node, add_build, add_tests)]
     if has_special_root_node:
         nodes = nodes[1:]
 
     dependencies = []
     if has_special_root_node:
         deps = {"ref": special_id,
-                "dependsOn": [f"pkg:conan/{d.dst.name}@{d.dst.ref.version}?rref={d.dst.ref.revision}"
-                              for d in graph.root.edges]}
+                "dependsOn": [_calculate_bomref(d.dst)
+                              for d in graph.root.edges
+                              if should_add_node(d.dst, add_build, add_tests)]}
         dependencies.append(deps)
     for c in nodes:
-        deps = {"ref": f"pkg:conan/{c.name}@{c.ref.version}?rref={c.ref.revision}"}
-        dep = [d for d in c.edges if (d.dst.context == "host" or add_build) and (not d.dst.test or add_tests)]
+        deps = {"ref": _calculate_bomref(c)}
+        dep = [d for d in c.edges if should_add_node(d.dst, add_build, add_tests)]
 
-        depends_on = [f"pkg:conan/{d.dst.name}@{d.dst.ref.version}?rref={d.dst.ref.revision}" for d in dep]
+        depends_on = [_calculate_bomref(d.dst) for d in dep
+                      if should_add_node(d.dst, add_build, add_tests)]
         if depends_on:
             deps["dependsOn"] = depends_on
         dependencies.append(deps)
@@ -146,7 +157,7 @@ def cyclonedx_1_6(conanfile, name=None, add_build=False, add_tests=False, **kwar
     sbom_cyclonedx_1_6 = {
         **({"components": [{
             **({"authors": [{"name": node.conanfile.author}]} if node.conanfile.author else {}),
-            "bom-ref": f"pkg:conan/{node.name}@{node.ref.version}?rref={node.ref.revision}",
+            "bom-ref": _calculate_bomref(node),
             "description": node.conanfile.description,
             **({"externalReferences": [{
                 "type": "website",
@@ -162,13 +173,13 @@ def cyclonedx_1_6(conanfile, name=None, add_build=False, add_tests=False, **kwar
         "metadata": {
             "component": {
                 **({"authors": [{"name": conanfile.author}]} if conanfile.author else {}),
-                "bom-ref": special_id if has_special_root_node else f"pkg:conan/{conanfile.name}@{conanfile.ref.version}?rref={conanfile.ref.revision}",
+                "bom-ref": special_id if has_special_root_node else _calculate_bomref(conanfile),
                 "name": name if name else name_default,
                 "type": "application" if conanfile.package_type == "application" else "library"
             },
             "timestamp": f"{datetime.fromtimestamp(time.time(), tz=timezone.utc).strftime('%Y-%m-%dT%H:%M:%SZ')}",
             "tools": {
-                "components":[{
+                "components": [{
                     "type": "application",
                     "name": "Conan-io",
                     "version": str(conan_version),
@@ -187,15 +198,24 @@ def _calculate_licenses(component):
     from conan.tools.sbom.spdx_licenses import NORMALIZED_VALID_SPDX_LICENSES
     licenses = component.conanfile.license
 
-    if isinstance(licenses, str): # Just one license
+    if isinstance(licenses, str):  # Just one license
         field = "id" if licenses.lower() in NORMALIZED_VALID_SPDX_LICENSES else "name"
-        return [{"license":{ field: licenses }}]
+        return [{"license": {field: licenses}}]
 
-    return [ # More than one license
+    return [
+        # More than one license
         {"license": {
-            "id" if l.lower() in NORMALIZED_VALID_SPDX_LICENSES else "name": l
+            "id" if lic.lower() in NORMALIZED_VALID_SPDX_LICENSES else "name": lic
         }}
-        for l in licenses
+        for lic in licenses
     ]
 
 
+def _calculate_bomref(component):
+    user = f"&user={component.ref.user}" if component.ref.user else ""
+    channel = f"&channel={component.ref.channel}" if component.ref.channel else ""
+    return f"pkg:conan/{component.name}@{component.ref.version}?rref={component.ref.revision}{user}{channel}"
+
+
+def should_add_node(node, add_build, add_tests):
+    return (node.context == "host" or add_build) and (not node.test or add_tests)
